% Generate the spatial part of the Quadratic Local Sctochastic Volatility operator 
% for the partition of unity setup given local computational nodes, pu weights, 
% interest rate, volatility, shape parameter and type of basis function

function [B,A0] = HHWop(x,puw,sig1,sig2,rho12,rho13,rho23,kappa,eta,a,b,phi,ep,ptype)
    
    rc = xcdist(x,x,1);
    n = size(x,1);
    
    A0 = RBFmat(phi,ep,rc,'0',1);
    
    Ax = RBFmat(phi,ep,rc,'1',1);
    Ay = RBFmat(phi,ep,rc,'1',2);
    Az = RBFmat(phi,ep,rc,'1',3);
    
    Axx = RBFmat(phi,ep,rc,'2',1);
    Ayy = RBFmat(phi,ep,rc,'2',2);
    Azz = RBFmat(phi,ep,rc,'2',3);
    
    Axy = RBFmat(phi,ep,rc,'m2',[1,2]);
    Axz = RBFmat(phi,ep,rc,'m2',[1,3]);
    Ayz = RBFmat(phi,ep,rc,'m2',[2,3]);

    Wi = spdiags(puw.w,0,n,n);

    Wix = spdiags(puw.wx,0,n,n);
    Wiy = spdiags(puw.wy,0,n,n);
    Wiz = spdiags(puw.wz,0,n,n);

    Wixx = spdiags(puw.wxx,0,n,n);
    Wiyy = spdiags(puw.wyy,0,n,n);
    Wizz = spdiags(puw.wzz,0,n,n);

    Wixy = spdiags(puw.wxy,0,n,n);
    Wixz = spdiags(puw.wxz,0,n,n);
    Wiyz = spdiags(puw.wyz,0,n,n);

    S = spdiags(x(:,1),0,n,n);
    V = spdiags(x(:,2),0,n,n);
    R = spdiags(x(:,3),0,n,n);

    D0 = Wi*A0; 
    
    Ds = Wi*Ax + Wix*A0;
    Dv = Wi*Ay + Wiy*A0;
    Dr = Wi*Az + Wiz*A0;
    
    Dss = Wi*Axx + 2*Wix*Ax + Wixx*A0; 
    Dvv = Wi*Ayy + 2*Wiy*Ay + Wiyy*A0;
    Drr = Wi*Azz + 2*Wiz*Az + Wizz*A0;
    
    Dsv = Wi*Axy + Wix*Ay + Wiy*Ax + Wixy*A0;
    Dsr = Wi*Axz + Wix*Az + Wiz*Ax + Wixz*A0;
    Dvr = Wi*Ayz + Wiy*Az + Wiz*Ay + Wiyz*A0; 


%     B = (0.5*S.^2.*V*Dss + 0.5*sig1^2*V*Dvv + 0.5*sig2^2*R.^(2*beta)*Drr ...
%         + rho12*sig1*S.*V*Dsv + rho13*sig2*S.*sqrt(V)*R.^(beta)*Dsr + rho23*sig1*sig2*sqrt(V)*R.^(beta)*Dvr ...
%         + R.*S*Ds + kappa*eta*Dv - kappa*V*Dv + a*b*Dr - a*R*Dr - R*D0);
    
    B = (0.5*S.^2.*V*Dss + 0.5*sig1^2*V*Dvv + 0.5*sig2^2*Drr ...
        + rho12*sig1*S.*V*Dsv + rho13*sig2*S.*sqrt(V)*Dsr + rho23*sig1*sig2*sqrt(V)*Dvr ...
        + R.*S*Ds + kappa*eta*Dv - kappa*V*Dv + a*b*Dr - a*R*Dr - R*D0);


    
    Bs = -R*D0; 
    
%     Bs = (0.5*sig1^2*V*Dvv + 0.5*sig2^2*Drr ...
%         + rho12*sig1*S.*V*Dsv + rho13*sig2*S.*sqrt(V)*Dsr + rho23*sig1*sig2*sqrt(V)*Dvr ...
%         + R.*S*Ds + kappa*eta*Dv - kappa*V*Dv + a*b*Dr - a*R*Dr - R*D0);
    
    pos = find(ptype==1);
        B(pos,:) = Bs(pos,:);
    pos = find(ptype==2);
       B(pos,:) = Bs(pos,:);

%     Bv = (0.5*S.^2.*V*Dss + 0.5*sig2^2*R.^(2*beta)*Drr ...
%         + rho13*sig2*S.*sqrt(V)*R.^(beta)*Dsr ...
%         + R.*S*Ds + a*b*Dr - a*R*Dr - R*D0);
    
    Bv = (0.5*S.^2.*V*Dss + 0.5*sig2^2*Drr ...
        + rho13*sig2*S.*sqrt(V)*Dsr ...
        + R.*S*Ds + a*b*Dr - a*R*Dr - R*D0);
    
    pos = find(ptype==3);
    %    B(pos,:) = Bv(pos,:);
    pos = find(ptype==4);
    %    B(pos,:) = Bv(pos,:);
    
    %Br = (0.5*S.^2.*V*Dss + 0.5*sig1^2*V*Dvv ...
    %    + rho12*sig1*S.*V*Dsv ...
    %    + R.*S*Ds + kappa*eta*Dv - kappa*V*Dv - R*D0);
    Br = (0.5*S.^2.*V*Dss + 0.5*sig1^2*V*Dvv ...
        + rho12*sig1*S.*V*Dsv + rho13*sig2*S.*sqrt(V)*Dsr + rho23*sig1*sig2*sqrt(V)*Dvr ...
        + R.*S*Ds + kappa*eta*Dv - kappa*V*Dv + a*b*Dr - a*R*Dr - R*D0);

    pos = find(ptype==5);
    %B(pos,:) = Br(pos,:);
    pos = find(ptype==6);
    %B(pos,:) = Br(pos,:);
    
    B = B/A0;








