function U = HHWpum(S,K0,T,R,V,kappa,eta,sig1,sig2,rho12,rho13,rho23,a,b,payoff,phi,ep,xc,M,xm,Rad,op,tind)

% Scale the strike to reduce to unit-strike problem
%K0 = K;
K = 1;

% Scale center points
%xc(:,1) = xc(:,1)/K0;
N = length(xc);
Np = size(xm,1);

xmin = min(xc(:,1));
xmax = max(xc(:,1));
vmin = min(xc(:,2));
vmax = max(xc(:,2));
rmin = min(xc(:,3));
rmax = max(xc(:,3));

bs0 = find(tind==1);
bsN = find(tind==2);

% Scale evaluation points
[ss,vv,rr]=meshgrid(S/K0,V,R);
xe = [ss(:),vv(:),rr(:)];
Ne = length(xe);

p = str2num(op);
fac = K0^(1-p);

% Compute the size of the partitions
%Hx = xmax/npx;
%x = xmin+Hx/2:Hx:xmax-Hx/2;
%Hv = (vmax-vmin)/npv;
%v = vmin+Hv/2:Hv:vmax-Hv/2;
%Hr = (rmax-rmin)/npr;
%z = rmin+Hr/2:Hr:rmax-Hr/2;

%Rad = sqrt(3)*max([Hx,Hv,Hr])/2*(1+delta);

%[xm,ym,zm] = meshgrid(x,v,z); % centers of partitions
%xm = [xm(:) ym(:) zm(:)];
%Np = length(xm); % number of partitions

% Check which point goes to which partition
for i = 1:Np
    flagin = sqrt((xm(i,1)-xc(:,1)).^2 + (xm(i,2)-xc(:,2)).^2 + (xm(i,3)-xc(:,3)).^2) <= Rad;
    ibox(i).ind = find(flagin);
    Ni(i)=length(ibox(i).ind);
end

% Compute the weight functions for each partition using Shepard's method
puw = puweightrbf3D(xc,ibox,xm,Rad);


% Do the same for eval points
for i = 1:Np
    flagin = sqrt((xm(i,1)-xe(:,1)).^2 + (xm(i,2)-xe(:,2)).^2 + (xm(i,3)-xe(:,3)).^2) <= Rad;
    ebox(i).ind = find(flagin);
    Nei(i)=length(ebox(i).ind);
end

% Compute the weight functions for each partition using Shepard's method
puw_e = puweightrbf3D(xe,ebox,xm,Rad);


% Form local RBF matrices
L = spalloc(N,N,sum(Ni.^2));
I = speye(N);
for i = 1:Np  
    
    ind = ibox(i).ind;
    xloc = xc(ind,:);
    ptype = tind(ind);
    % Compute the HHW spatial operator
    [B,A0] = HHWop(xloc,puw(i),sig1,sig2,rho12,rho13,rho23,kappa,eta,a,b,phi,ep,ptype);
    L(ind,ind) = L(ind,ind) + B;  
  
    ind_e = ebox(i).ind;
    flagempt = isempty(ind_e);
    if flagempt == 0
        re = xcdist(xe(ind_e,:),xloc,op);
        Wei = spdiags(puw_e(i).w,0,Nei(i),Nei(i));
        E{i} = Wei*RBFmat(phi,ep,re,op)/A0; % Evaluation matrix
    end
    
end

% Coefficients for BDF-2
[k,beta0,beta1,beta2] = BDF2coeffs(T,M);
  
% Assemble full operator
C = I - beta0*L;

% Set initial condition
tn = k(1);
u0 = real(payoff(xc(:,1),K,xc(:,3),0));
rhs = u0;

% iLU setting
setup.type = 'nofill';
[iL,iU] = ilu(C,setup);
tol = 1e-04;
maxit = 10;

% Time-stepping loop 
for n = 1:M
   
    % New solution value from linear system
    [u,flag,relres,iter,resvec] = gmres(C,rhs,[],tol,maxit,iL,iU,u0);

    % Prepare the right hand side for the next step
    nextstep = min(M,n+1);
    tn = sum(k(1:nextstep)); % Should be one step ahead        
    rhs = beta1(nextstep)*u-beta2(nextstep)*u0;
  
    % BC
    rhs(bs0) = 0;
    rhs(bsN) = payoff(xmax,K,xc(bsN,3),tn);
    
    % Update
    u0 = u;
    
end

% Compute the solution or some derivative at the evaluation points
U = zeros(Ne,1);
for i=1:Np
    ind = ibox(i).ind;
    ind_e = ebox(i).ind;
    flagempty = isempty(ind_e);
    if flagempty == 0
        U(ind_e) = U(ind_e) + E{i}*u(ind);
    end
end  

% Scale back
U = fac*U;





