function [U]=HSTdnTouch_RBFPUM(tol)

alpha = 0;
beta = 1;
gamma = 0;
T = 1; % Time to maturity
rho = -0.36;
kappa = 2.58;
eta = 0.043;
sig = 1; 
r = 0; % Interest rate
K0 = 100; % Strike price
K = 1;
S = [75, 100, 125]; % Evaluation point
V = 0.114;
low = 50;
up = 150;
payoff = @(S) double(low/K0<S & S<up/K0); % Payoff function

phi = 'mq'; % RBF type
delta = 0.2;
op = '0';

% Domain boundaries
xmin = low/K0; %.1*K; % Lower domain boundary in stock
xmax = up/K0; % Upper domain boundary in stock
ymin = 0; % Lower domain boundary in vol
ymax = 0.7; % Upper domain boundary in vol

% npx ep maxerr
% 12   8 0.17
% 16   8 0.19 % For 18 and 20, ep=8 becomes ill-cond
% 12  20 0.23 % With clusterpts
% 16  60 0.19
% 20  80 0.165
% 40 100 0.25
%  
npx = max(2,round(-4.33*log(tol)-16.93)); % # Partitions in stock direction
ep = 0.8889*npx+1.4444; % Shape parameter
Nx = 6*npx; % # Space points in stock direction
Hx = (xmax-xmin)/npx;
npy = ceil((ymax-ymin)/Hx);
Ny = max(6,Nx/npx*npy);

M = 10; % # Time steps

% Generate computational nodes
x = linspace(xmin,xmax,Nx);
ell=0.25;
%x = ClusterPts(xmin,xmax,Nx,[low/K0 up/K0],ell,'sinh');
%x = ClusterPts(xmin,xmax,Nx,[low/K0 ],ell,'sinh');
Nx = length(x);
y = linspace(ymin,ymax,Ny);
[xx,yy] = meshgrid(x,y);

xtype = zeros(size(yy));
xtype(1,:) = 1; % V-boundary
xtype(end,:) = 2; % V-boundary
xtype(2:end-1,1) = 3; % S-boundary
xtype(2:end-1,end) = 4; % S-boundary
xc = [xx(:),yy(:)];

Hx = (xmax-xmin)/npx;
x = xmin+Hx/2:Hx:xmax-Hx/2;
Hy = (ymax-ymin)/npy;
y = ymin+Hy/2:Hy:ymax-Hy/2;
[xm,ym] = meshgrid(x,y); % centers of partitions
xm = [xm(:) ym(:)];
Np = length(xm); % number of partitions
R = sqrt(2)*max([Hx,Hy])/2*(1+delta);


%tic
U = QLSVpumDNT(S/K0,K0,T,r,V,kappa,eta,sig,rho,alpha,beta,gamma,payoff,phi,ep,xc,M,xm,R,op,xtype,Ny,Nx);
U = U';
%toc

%Uref = [0.8345, 0.8998, 0.6684]

%err = abs(Uref-U)

%relerr = err./Uref


end

    












