function [U]=QLSVdnTouch_RBFPUM(tol)
%
% Note that the boundary conditions should be zero at the upper and lower barriers.
%  
alpha = 0.02;
beta = 0;
gamma = 0;
T = 1; % Time to maturity
rho = -0.36;
kappa = 2.58;
eta = 0.043;
sig = 1; 
r = 0; % Interest rate
K0 = 100; % Strike price
K = 1;
S = [75, 100, 125]; % Evaluation point
V = 0.114;
low = 50;
up = 150;
payoff = @(S) double(low/K0<S & S<up/K0); % Payoff function

phi = 'mq'; % RBF type
delta = 0.2;
op = '0';

% Domain boundaries
xmin = low/K0;0; %.1*K; % Lower domain boundary in stock
xmax = up/K0; 4*K; % Upper domain boundary in stock
ymin = 0; % Lower domain boundary in vol
ymax = 0.7; % Upper domain boundary in vol

% npx ep maxerr with cluster, no cluster
%  2  12 0.056 0.050
%      4 0.11  0.02 *
%      8 0.03 *0.036
%      7 0.04  0.032
%  3   8 0.04
%      9 0.04
%      7 0.03 
%    4-5 0.02 * 0.02
%      5        0.01*
%  4   7 0.03  
%    4-5 0.02 *
%    5-6        0.01 
%  5   5 0.015    0.0077
%      6 0.0147 * 0.0078 
%  6   7 0.01 * 0.0052
%      6        0.0052 
%  8  10 0.009
%      8        0.0072 *
% 10  10 0.009
%      9        0.0041 *
% 12  14        0.0056
%  
% 16  20        0.01 ca
npx = max(2,round(-4.33*log(tol)-16.93)); % # Partitions in stock direction
ep = 0.8889*npx+1.4444; % Shape parameter
Nx = 6*npx; % # Space points in stock direction
Hx = (xmax-xmin)/npx;
npy = ceil((ymax-ymin)/Hx);
Ny = max(6,Nx/npx*npy);

M = 10; % # Time steps

% Generate computational nodes
x = linspace(xmin,xmax,Nx);
ell=0.25;
%x = ClusterPts(xmin,xmax,Nx,[low/K0 up/K0],ell,'sinh');
%x = ClusterPts(xmin,xmax,Nx,[low/K0 ],ell,'sinh');
Nx = length(x);
y = linspace(ymin,ymax,Ny);
[xx,yy] = meshgrid(x,y);

xtype = zeros(size(yy));
xtype(1,:) = 1; % V-boundary
xtype(end,:) = 2; % V-boundary
xtype(2:end-1,1) = 3; % S-boundary
xtype(2:end-1,end) = 4; % S-boundary
xc = [xx(:),yy(:)];

Hx = (xmax-xmin)/npx;
x = xmin+Hx/2:Hx:xmax-Hx/2;
Hy = (ymax-ymin)/npy;
y = ymin+Hy/2:Hy:ymax-Hy/2;
[xm,ym] = meshgrid(x,y); % centers of partitions
xm = [xm(:) ym(:)];
Np = length(xm); % number of partitions
R = sqrt(2)*max([Hx,Hy])/2*(1+delta);


%tic
U = QLSVpumDNT(S/K0,K0,T,r,V,kappa,eta,sig,rho,alpha,beta,gamma,payoff,phi,ep,xc,M,xm,R,op,xtype,Ny,Nx);
U = U';
%toc

%Uref = [0.9338 0.9148 0.5930]

%err = abs(Uref-U)

%relerr = err./Uref


end

    












