% If we decide to scale things, can we do it here or does it need to be
% inside?
% We scale the grids with K already here, and assume that QLSV-operator
% solves the problem with K as a scaling, and unit as default.
%
function [U]=QLSVeuCall_RBFPUM(tol)
%
% Given problem parameters and functions
%  
alpha = 0.02;
beta = 0;
gamma = 0;
T = 1; % Time to maturity
rho = -0.36;
kappa = 2.58;
eta = 0.043;
sig = 1; 
r = 0; % Interest rate
K0 = 100; % Strike price
K = 1; % The strike price we compute with
S = [75, 100, 125]; % Evaluation point
V = 0.114;
payoff = @(S,K,r,t) max(S-K*exp(-r*t),0); % Payoff function
%
% Method parameters that will not change
%
phi = 'mq'; % RBF type
delta = 0.2;
op = '0';
%
% Parameters to choose as a function of tol
%
% Domain boundaries
xmin = 0; % Lower domain boundary in stock
xmax = 3*K; % Upper domain boundary in stock
ymin = 0; % Lower domain boundary in vol
ymax = 0.7; % Upper domain boundary in vol

npx = max(1,round(-(log(tol)+0.91)/0.23)); % # Partitions in stock direction
ep = max((2+npx/5),0.64*10^(0.054*npx)); % Shape parameter
Nx = 6*npx; % # Space points in stock direction
Hx = (xmax-xmin)/npx; % Will scale with K later
npy = ceil((ymax-ymin)/Hx);
Ny = max(6,Nx/npx*npy); % Previously used 0.5 of the x-direction

M = 20; % # Time steps

% Generate computational nodes
x = linspace(xmin,xmax,Nx);
%ell=1;
%x = ClusterPts(xmin,xmax,Nx,K,ell,'sinh');
%figure(1),plot(x,0*x,'o')
y = linspace(ymin,ymax,Ny);
[xx,yy] = meshgrid(x,y);

xtype = zeros(size(yy));
xtype(1,:) = 1; % V-boundary
xtype(end,:) = 2; % V-boundary
xtype(2:end-1,1) = 3; % S-boundary
xtype(2:end-1,end) = 4; % S-boundary
xc = [xx(:),yy(:)];

% Generate patches
Hx = (xmax-xmin)/npx;
x = xmin+Hx/2:Hx:xmax-Hx/2;
Hy = (ymax-ymin)/npy;
y = ymin+Hy/2:Hy:ymax-Hy/2;
[xm,ym] = meshgrid(x,y); % centers of partitions
xm = [xm(:) ym(:)];
Np = length(xm); % number of partitions
R = sqrt(2)*max([Hx,Hy])/2*(1+delta);


%tic
U = QLSVpum(S/K0,K0,T,r,V,kappa,eta,sig,rho,alpha,beta,gamma,payoff,phi,ep,xc,M,xm,R,op,xtype);
U = U';
%toc

%Uref = [0.5274 8.9023 29.1598]

%err = abs(Uref-U)

%relerr = err./Uref















