% Generate the spatial part of the Quadratic Local Sctochastic Volatility operator 
% for the partition of unity setup given local computational nodes, pu weights, 
% interest rate, volatility, shape parameter and type of basis function

function [B,A0] = QLSVoperator(x,puw,r,sig,rho,kappa,eta,alpha,beta,gamma,qlsv,phi,ep,ptype)

    n = size(x,1);
    rc = xcdist(x,x,1);

    A0  = RBFmat(phi,ep,rc,'0');
    Ax  = RBFmat(phi,ep,rc,'1',1);
    Ay  = RBFmat(phi,ep,rc,'1',2);
    Axx = RBFmat(phi,ep,rc,'2',1);
    Axy = RBFmat(phi,ep,rc,'m2',[1,2]);
    Ayy = RBFmat(phi,ep,rc,'2',2);

    Wi = spdiags(puw.w,0,n,n);
    Wix = spdiags(puw.wx,0,n,n);
    Wiy = spdiags(puw.wy,0,n,n);
    Wixx = spdiags(puw.wxx,0,n,n);
    Wixy = spdiags(puw.wxy,0,n,n);
    Wiyy = spdiags(puw.wyy,0,n,n);

    X = spdiags(x(:,1),0,n,n);
    Y = spdiags(x(:,2),0,n,n);

    D0 = Wi*A0; 
    Dx = Wi*Ax + Wix*A0;
    Dy = Wi*Ay + Wiy*A0;

    Dxx = Wi*Axx + 2*Wix*Ax + Wixx*A0; 
    Dyy = Wi*Ayy + 2*Wiy*Ay + Wiyy*A0;

    Dxy = Wi*Axy + Wix*Ay + Wiy*Ax + Wixy*A0;

    f = qlsv(x(:,1),alpha,beta,gamma);

    F = spdiags(f,0,n,n);

    B = (0.5*F.^2.*Y*Dxx + rho*sig*F.*Y*Dxy + 0.5*sig^2*Y*Dyy + r*X*Dx ...
    + kappa*eta*Dy - kappa*Y*Dy - r*D0)/A0;

    By = (0.5*F.^2.*Y*Dxx + r*X*Dx - r*D0)/A0;
    
    Bx = (0.5*sig^2*Y*Dyy + r*X*Dx + kappa*eta*Dy - kappa*Y*Dy - r*D0)/A0;
    
    pos = find(ptype==1);
    %B(pos,:) = By(pos,:);  
    pos = find(ptype==2);
    %B(pos,:) = By(pos,:);
    %pos = find(ptype==4);
    %B(pos,:) = Bx(pos,:);








