% Action plan: 
% 0. Start without scaling. This could be very strange with the variable
% volatility and would need a methematical investigation.
% 1. Harmonize this code with the SABR-code as a template. Especially check that boundary conditions are applied in the
% correct dimensions. 
% 2. Then update each main function such that it takes a continuous
% tolerance dependence. Check if 2K is really enough for the upper
% boundary or if it gives bias also here.
% 3. What is a good number of points per patch?


% What is the boundary condition for the spread option?
function U = QLSVpum(S,K0,T,r,V,kappa,eta,sig,rho,alpha,beta,gamma,payoff,phi,ep,xc,M,xm,R,op,xtype)

% Scale the strike to reduce to unit-strike problem
K=1;

% Scale center points. The points come in scaled
%xc(:,1) = xc(:,1)/K0;
%xc(:,2) = xc(:,2);
N = size(xc,1);
Np = size(xm,1);
xmax = max(xc(:,1));

% Scale evaluation points. They also come in scaled
%[ss,vv]=meshgrid(S/K0,V);
[ss,vv]=meshgrid(S,V);
xe = [ss(:),vv(:)];
Ne = size(xe,1);

% Some stuff for scaling back
p = str2num(op);
fac = K0^(1-p);

% Quadratic Local Stochastic Volatility function
%qlsv = @(S,alpha,beta,gamma,sc) 0.5*alpha*sc^2*S.^2 + beta*sc*S +
%gamma*ones(size(S)); 
qlsv = @(S,alpha,beta,gamma) (0.5*alpha*(K0*S).^2 + beta*K0*S + gamma*ones(size(S)))/K0; 

% Check which point goes to which partition
for i=1:Np
    flagin = sqrt((xm(i,1)-xc(:,1)).^2 + (xm(i,2)-xc(:,2)).^2) <= R;
    ibox(i).ind = find(flagin);
    Ni(i)=length(ibox(i).ind);
    %
    flagin = sqrt((xm(i,1)-xe(:,1)).^2 + (xm(i,2)-xe(:,2)).^2) <= R;
    ebox(i).ind = find(flagin);
    Nei(i)=length(ebox(i).ind);
end

% Compute the weight functions for each partition using Shepard's method
puw = puweightrbf(xc,ibox,xm,R);
epatch = find(Nei>0);
puw_e = puweightrbf(xe,ebox(epatch),xm(epatch,:),R);
puw_e(epatch)=puw_e;

% Form local RBF matrices
L = spalloc(N,N,sum(Ni.^2));
for i=1:Np    
    ind = ibox(i).ind;
    xloc = xc(ind,:);
    ptype = xtype(ind);
    
    % Compute the BS spatial operator
    [B,A0] = QLSVoperator(xloc,puw(i),r,sig,rho,kappa,eta, ...
                          alpha,beta,gamma,qlsv,phi,ep,ptype);   
    L(ind,ind) = L(ind,ind) + B;  
    if (Nei(i)>0)
      ind_e = ebox(i).ind;
      re = xcdist(xe(ind_e,:),xloc,op);
      Wei = spdiags(puw_e(i).w,0,Nei(i),Nei(i));
      E(i).E = Wei*RBFmat(phi,ep,re,op)/A0; % Evaluation matrix
    end    
end
 
% Coefficients for BDF-2
[k,beta0,beta1,beta2]=BDF2coeffs(T,M);
  
% Assemble full operator
I = speye(N);
C = I - beta0*L;

bx0 = find(xtype==3);
bxN = find(xtype==4);

C([bx0;bxN],:) = I([bx0;bxN],:);

% Set initial condition
tn = k(1);
u0 = payoff(xc(:,1),K,r,0);

rhs(:,1) = u0;
rhs(bx0,1) = 0;
rhs(bxN,1) = payoff(xmax,K,r,tn);

setup.type = 'nofill';
[iL,iU] = ilu(C,setup);
tol = 1e-6;
maxit = 15;

% Time-stepping loop
for n = 1:M
    
    % New solution value from linear system
    [u,flag,relres,iter,resvec] = gmres(C,rhs,[],tol,maxit,iL,iU,u0);
    
    % Prepare the right hand side for the next step
    nextstep = min(M,n+1);
    tn = sum(k(1:nextstep)); % Should be one step ahead        
    rhs = beta1(nextstep)*u-beta2(nextstep)*u0;
    
    % Boundary conditions
    rhs(bx0) = 0;
    rhs(bxN) = payoff(xmax,K,r,tn);
        
    % Update
    u0 = u;
end
%figure(2),mesh(reshape(xc(:,1),36,144),reshape(xc(:,2),36,144),reshape(u,36,144))
% Compute the solution or some derivative at the evaluation points
U = zeros(Ne,1);
for i=epatch
    ind = ibox(i).ind;
    ind_e = ebox(i).ind;
    U(ind_e) = U(ind_e) + E(i).E*u(ind);
end  

% Scale back
U = U*fac;







