% What is the boundary condition for the spread option?
% Can we enter the low and up as the payoff? I would assume so
% payoff is not defined here
function U = QLSVpumDNT(S,K0,T,r,V,kappa,eta,sig,rho,alpha,beta,gamma,payoff,phi,ep,xc,M,xm,R,op,xtype,Ny,Nx)

% Scale the strike to reduce to unit-strike problem
%K0 = K;
K = 1;
%up = up/K0;
%low = low/K0;
  
% Scale center points
%xc(:,1) = xc(:,1)/K0;
%xc(:,2) = xc(:,2);
N = size(xc,1);
Np = size(xm,1);

% Scale evaluation points
%[ss,vv]=meshgrid(S/K0,V);
[ss,vv]=meshgrid(S,V);
xe = [ss(:),vv(:)];
Ne = size(xe,1);

% Some stuff for scaling back
p = str2num(op);
fac = K0^(-p);

% Find boundaries
%xmin = min(xc(:,1));
%xmax = max(xc(:,1));
%ymin = min(xc(:,2));
%ymax = max(xc(:,2));
%bx0 = find(xc(:,1) == xmin);
%bxN = find(xc(:,1) == xmax);

% Some stuff for scaling back
p = str2num(op);
fac = 1; %K0^(1-p);

% Quadratic Local Stochastic Volatility function
qlsv = @(S,alpha,beta,gamma) (0.5*alpha*(K0*S).^2 + beta*K0*S + gamma)/K0; 

% Compute the size of the partitions
%Hx = (xmax-xmin)/npx;
%x = xmin+Hx/2:Hx:xmax-Hx/2;
%Hy = (ymax-ymin)/npy;
%y = ymin+Hy/2:Hy:ymax-Hy/2;

%R = sqrt(2)*max([Hx,Hy])/2*(1+delta);

%[xm,ym] = meshgrid(x,y); % centers of partitions
%xm = [xm(:) ym(:)];
%Np = length(xm); % number of partitions

% Check which point goes to which partition
for i=1:Np
    flagin = sqrt((xm(i,1)-xc(:,1)).^2 + (xm(i,2)-xc(:,2)).^2) <= R;
    ibox(i).ind = find(flagin);
    Ni(i)=length(ibox(i).ind);
    %
    flagin = sqrt((xm(i,1)-xe(:,1)).^2 + (xm(i,2)-xe(:,2)).^2) <= R;
    ebox(i).ind = find(flagin);
    Nei(i)=length(ebox(i).ind);
end

% Compute the weight functions for each partition using Shepard's method
puw = puweightrbf(xc,ibox,xm,R);
epatch = find(Nei>0);
puw_e = puweightrbf(xe,ebox(epatch),xm(epatch,:),R);
puw_e(epatch)=puw_e;

% Form local RBF matrices
L = spalloc(N,N,sum(Ni.^2));
for i=1:Np    
    ind = ibox(i).ind;
    xloc = xc(ind,:);
    ptype = xtype(ind);

    % Compute the BS spatial operator
    [B,A0] = QLSVoperator(xloc,puw(i),r,sig,rho,kappa,eta, ...
                          alpha,beta,gamma,qlsv,phi,ep,ptype);   
    L(ind,ind) = L(ind,ind) + B;  
    if (Nei(i)>0)
      ind_e = ebox(i).ind;
      re = xcdist(xe(ind_e,:),xloc,op);
      Wei = spdiags(puw_e(i).w,0,Nei(i),Nei(i));
      E(i).E = Wei*RBFmat(phi,ep,re,op)/A0; % Evaluation matrix
    end    
end
 

% Coefficients for BDF-2
[k,beta0,beta1,beta2]=BDF2coeffs(T,M);
  
% Assemble full operator
I = speye(N);
C = I - beta0*L;

bx0 = find(xtype==3);
bxN = find(xtype==4);

C([bx0;bxN],:) = I([bx0;bxN],:);

% Set initial condition
tn = k(1);
%u0 = payoff(xc(:,1),low,up);
u0 = payoff(xc(:,1));

rhs(:,1) = u0;
rhs(bx0,1) = 0;
rhs(bxN,1) = 0; % Zero boundary conditions on both sides is a new thing

setup.type = 'nofill';
[iL,iU] = ilu(C,setup);
tol = 1e-06;
maxit = 15;

% Time-stepping loop
for n = 1:M
    
    % New solution value from linear system
    [u,flag,relres,iter,resvec] = gmres(C,rhs,[],tol,maxit,iL,iU,u0);

    % Prepare the right hand side for the next step
    nextstep = min(M,n+1);
    tn = sum(k(1:nextstep)); % Should be one step ahead        
    rhs = beta1(nextstep)*u-beta2(nextstep)*u0;
    
    % Boundary conditions
    rhs(bx0) = 0;
    rhs(bxN) = 0;
        
    % Update
    u0 = u;
end
%xx=reshape(xc(:,1),Ny,Nx);
%yy=reshape(xc(:,2),Ny,Nx);
%zz=reshape(u,Ny,Nx);
%figure(2),clf
%mesh(xx,yy,zz)

% Compute the solution or some derivative at the evaluation points
U = zeros(Ne,1);
for i=epatch
    ind = ibox(i).ind;
    ind_e = ebox(i).ind;
    U(ind_e) = U(ind_e) + E(i).E*u(ind);
end  

% Scale back
U = U*fac;
end

%function F = payoff(S,L,U)
%    F = zeros(size(S));
%    ind = find(L<S & S<U);
%    F(ind) = 1;
%end
