function Ue = ...
      SABRpum(S,KK,T,r,V,rho,alpha,beta,payoff,phi,ep,xc,M,xm,R,op,xtype)

N = size(xc,1);
Np = size(xm,1); % number of partitions
xmax = max(xc(:,1));

% Evaluation points (no scaling)
[ss,vv]=meshgrid(S,V);
xe = [ss(:),vv(:)];
Ne = size(xe,1);

% Check which point goes to which partition
for i=1:Np
    flagin = sqrt((xm(i,1)-xc(:,1)).^2 + (xm(i,2)-xc(:,2)).^2) <= R;
    ibox(i).ind = find(flagin);
    Ni(i)=length(ibox(i).ind);
    %
    flagin = sqrt((xm(i,1)-xe(:,1)).^2 + (xm(i,2)-xe(:,2)).^2) <= R;
    ebox(i).ind = find(flagin);
    Nei(i)=length(ebox(i).ind);
end

% Compute the weight functions for each partition using Shepard's method
puw = puweightrbf(xc,ibox,xm,R);
epatch = find(Nei>0);
puwe = puweightrbf(xe,ebox(epatch),xm(epatch,:),R);
puwe(epatch)=puwe;

% Form local RBF matrices
L = spalloc(N,N,sum(Ni.^2));
for i=1:Np    
    ind = ibox(i).ind;
    xloc = xc(ind,:);
    ptype = xtype(ind);
    
    % Compute the BS spatial operator
    [B,A0] = SABRoperator(xloc,puw(i),r,rho,alpha,beta,phi,ep,ptype);   
    L(ind,ind) = L(ind,ind) + B;  
    
    if (Nei(i)>0)
      ind_e = ebox(i).ind;
      re = xcdist(xe(ind_e,:),xloc,op);
      Wei = spdiags(puwe(i).w,0,Nei(i),Nei(i));
      E(i).E = Wei*RBFmat(phi,ep,re,op)/A0; % Evaluation matrix
    end  
end    

% Coefficients for BDF-2
[k,beta0,beta1,beta2]=BDF2coeffs(T,M);
  
% Assemble full operator
I = speye(N);
C = I - beta0*L;

bx0 = find(xtype==3);
bxN = find(xtype==4);

C([bx0;bxN],:) = I([bx0;bxN],:);

for ii = 1:length(KK)
    K = KK(ii);
    
    % Set initial condition
    tn = k(1);
    u0 = payoff(xc(:,1),K,r,0);
    rhs(:,1) = u0;
    rhs(bx0,1) = 0;
    rhs(bxN,1) = payoff(xmax,K,r,tn);

    setup.type = 'nofill';
    [iL,iU] = ilu(C,setup);
    tol = 1e-8;
    maxit = 50;

    % Time-stepping loop
    for n = 1:M
    
      % New solution value from linear system
      [u,flag,relres,iter,resvec] = gmres(C,rhs,[],tol,maxit,iL,iU,u0);

      % Prepare the right hand side for the next step
      nextstep = min(M,n+1);
      tn = sum(k(1:nextstep)); % Should be one step ahead        
      rhs = beta1(nextstep)*u-beta2(nextstep)*u0;
    
      % Boundary conditions
      rhs(bx0) = 0;
      rhs(bxN) = payoff(xmax,K,r,tn);
        
      % Update
      u0 = u;
    end

    % Compute the solution or some derivative at the evaluation points
    U = zeros(Ne,1);
    for i=epatch
      ind = ibox(i).ind;
      ind_e = ebox(i).ind;
      U(ind_e) = U(ind_e) + E(i).E*u(ind);
    end  

    Ue(1,ii)=U;

end



